@@ -5,6 +5,7 @@ from datetime import datetime
 from datetime import timedelta
 from typing import List
 
+from asgiref.sync import sync_to_async
 import aiohttp
 from django.core.management.base import BaseCommand
 from django.db import OperationalError
@@ -27,7 +28,7 @@ class Command(BaseCommand):
     process_butch = settings.PUSH_NOTIFICATIONS.get('PROCESS_BUTCH', 100)
 
     def handle(self, *args, **kwargs):
-        self.stdout.write("push notifications emitter started")
+        print("push notifications emitter started")
 
         while True:
             interval = (datetime.utcnow() - timedelta(seconds=self.retry_interval)).replace(tzinfo=utc)
@@ -38,12 +39,12 @@ class Command(BaseCommand):
                         Q(last_retried_at__isnull=True) | Q(last_retried_at__lt=interval)
                     ).all()[:self.process_butch]
 
-                    events_len = len(events)
+                    events_len = events.count()
                     if events_len < 1:
                         time.sleep(1)
                         continue
 
-                    self.stdout.write(
+                    print(
                         "proccessing {} events".format(events_len)
                     )
 
@@ -52,23 +53,29 @@ class Command(BaseCommand):
                 traceback.print_exc()
             time.sleep(1)
 
+    # this needs to be separated like so because transactions do not support async mode in django 4.1
+    # https://docs.djangoproject.com/en/4.1/topics/async/#queries-the-orm
+    def create_event_tasks(self, session, events):
+        tasks = []
+        with transaction.atomic():
+            for event in events:
+                tasks.append(
+                    self.emit(session, event)
+                )
+        return tasks
+
     async def emits(self, events: List[PushNotificationEvent]):
         conn = aiohttp.TCPConnector(limit_per_host=self.connection_per_host)
         async with aiohttp.ClientSession(
                 conn_timeout=self.connection_timeout, read_timeout=self.read_timeout,
                 connector=conn) as session:
-            tasks = []
-            for event in events:
-                tasks.append(
-                    self.emit(session, event)
-                )
+            tasks = await sync_to_async(self.create_event_tasks)(session, events)
             await asyncio.gather(*tasks)
 
     async def emit(self, session: aiohttp.ClientSession, event: PushNotificationEvent):
-        event_id = event.event.event_id
-        self.stdout.write(
-            "emit event {}".format(event_id))
-        
+        print(
+            "emit event {}".format(event.event_id))
+
         utcnow = datetime.utcnow().replace(tzinfo=utc)
         event.retry = 1 if not event.retry else event.retry + 1
         event.last_retried_at = utcnow
@@ -77,14 +84,14 @@ class Command(BaseCommand):
             res = await event.post(session)
             res.raise_for_status()
             event.processed_at = utcnow
-            self.stdout.write(
+            print(
                 "event {} has been delivered to {}".format(
-                    event.event.event_id, res.url)
+                    event.event_id, res.url)
             )
         except Exception as e:
-            self.stderr.write(
+            print(
                 "error while delivering the event {}, retry {}: {}".format(
-                    event.event.event_id, event.retry, str(e))
+                    event.event_id, event.retry, str(e))
             )
         finally:
-            event.save()
+            await sync_to_async(event.save)()
