@@ -110,9 +110,12 @@ class PushNotificationEvent(models.Model):
             "User-Agent": "SpotiiPush/1.0 (https://www.spotii.me/bot.html)",
         }
 
+        push_notification = await PushNotification.actual_objects.aget(pk=self.push_notification_id)
+        event = await Event.objects.aget(pk=self.event_id)
+
         async with session.post(
-            self.push_notification.callback_url,
-            json=self.event.data(),
+            push_notification.callback_url,
+            json=event.data(),
             headers=headers
         ) as res:
             return res
